@@ -1,3 +1,4 @@
+from jet_bridge_base.utils.queryset import get_session_engine
 from sqlalchemy import text, select, column, func, desc, or_, cast
 from sqlalchemy import sql
 from sqlalchemy.sql import sqltypes
@@ -61,6 +62,7 @@ class SqlSerializer(Serializer):
     group = GroupSerializer(required=False)
     groups = GroupsSerializer(required=False)
     timezone = fields.CharField(required=False)
+    schema = fields.CharField(required=False)
     params = SqlParamsSerializers(required=False)
     params_obj = fields.JSONField(required=False)
     v = fields.IntegerField(default=1)
@@ -138,6 +140,8 @@ class SqlSerializer(Serializer):
 
     def filter_queryset(self, queryset, data):
         filters_instances = []
+        request = self.context.get('request')
+        session = request.session
 
         for item in data.get('columns', []):
             query_type = map_to_sql_type(item['data_type'])()
@@ -152,22 +156,32 @@ class SqlSerializer(Serializer):
                     )
                     filters_instances.append(instance)
 
-        def get_filter_value(name):
+        def get_filter_value(name, filters_instance=None):
             filter_items = list(filter(lambda x: x['name'] == name, data.get('filters', [])))
-            return filter_items[0]['value'] if len(filter_items) else None
+
+            if not len(filter_items):
+                return
+
+            value = filter_items[0]['value']
+
+            if filters_instance and value is not None and get_session_engine(session) == 'bigquery':
+                python_type = filters_instance.column.type.python_type
+                value = python_type(value)
+
+            return value
 
         for item in filters_instances:
             if item.name:
                 argument_name = '{}__{}'.format(item.name, item.lookup)
                 if item.exclude:
                     argument_name = 'exclude__{}'.format(argument_name)
-                value = get_filter_value(argument_name)
+                value = get_filter_value(argument_name, item)
 
                 if value is None and item.lookup == lookups.DEFAULT_LOOKUP:
                     argument_name = item.name
                     if item.exclude:
                         argument_name = 'exclude__{}'.format(argument_name)
-                    value = get_filter_value(argument_name)
+                    value = get_filter_value(argument_name, item)
             else:
                 value = None
 
@@ -182,10 +196,10 @@ class SqlSerializer(Serializer):
 
                 if isinstance(query_type, (sqltypes.Integer, sqltypes.Numeric)):
                     return cast(field, sqltypes.String).__eq__(search)
+                elif isinstance(query_type, (sqltypes.JSON, sqltypes.Enum)):
+                    return cast(field, sqltypes.String).ilike('%{}%'.format(search))
                 elif isinstance(query_type, sqltypes.String):
                     return field.ilike('%{}%'.format(search))
-                elif isinstance(query_type, sqltypes.JSON):
-                    return cast(field, sqltypes.String).ilike('%{}%'.format(search))
 
             operators = list(filter(lambda x: x is not None, map(map_column, data.get('columns', []))))
             queryset = queryset.filter(or_(*operators))
@@ -239,6 +253,13 @@ class SqlSerializer(Serializer):
                 session.rollback()
                 pass
 
+        if 'schema' in data:
+            try:
+                session.execute('SET search_path TO :schema', {'schema': data['schema']})
+            except SQLAlchemyError:
+                session.rollback()
+                pass
+
         subquery = text(query).columns().subquery('__jet_q2')
         count_rows = None
 
@@ -289,22 +310,23 @@ class SqlSerializer(Serializer):
             def map_row(row):
                 return list(map(lambda x: map_row_column(row[x]), row.keys()))
 
-            def map_column_description(column):
-                name = column.name if hasattr(column, 'name') else ''
-                sql_type = type_code_to_sql_type.get(column.type_code) if hasattr(column, 'type_code') else None
-                return name, {
-                    'field': sql_to_map_type(sql_type) if sql_type else None
-                }
-
-            type_code_to_sql_type = get_type_code_to_sql_type(request)
-            column_descriptions = dict(map(map_column_description, result.cursor.description))
-
+            cursor_description = result.cursor.description
             response = {
                 'data': list(map(map_row, result)),
-                'columns': list(map(map_column, result.keys())),
-                'column_descriptions': column_descriptions
+                'columns': list(map(map_column, result.keys()))
             }
 
+            type_code_to_sql_type = get_type_code_to_sql_type(request)
+            if type_code_to_sql_type:
+                def map_column_description(column):
+                    name = column.name if hasattr(column, 'name') else ''
+                    sql_type = type_code_to_sql_type.get(column.type_code) if hasattr(column, 'type_code') else None
+                    return name, {
+                        'field': sql_to_map_type(sql_type) if sql_type else None
+                    }
+
+                response['column_descriptions'] = dict(map(map_column_description, cursor_description))
+
             if count_rows is not None:
                 response['count'] = count_rows
 
